/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        AbsRequest: parent class for all CEMonitor requests

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: AbsRequest.h,v 1.38 2007/12/14 11:02:37 adorigo Exp $
*/

#ifndef __GLITE_CE_MONITOR_CLIENT_ABSREQUEST__
#define __GLITE_CE_MONITOR_CLIENT_ABSREQUEST__

#include "glite/ce/monitor-client-api-c/monitor_client_CEMON_CLIENTStub.h"
#include "glite/ce/monitor-client-api-c/GenericException.h"
#include "glite/ce/monitor-client-api-c/TopicNotSupportedException.h"
#include "glite/ce/monitor-client-api-c/DialectNotSupportedException.h"
#include "glite/ce/monitor-client-api-c/SubscriptionException.h"
#include "glite/ce/monitor-client-api-c/ServiceNotFoundException.h"
#include "glite/ce/monitor-client-api-c/GeneralException.h"
#include "glite/ce/monitor-client-api-c/AuthenticationException.h"
#include "glite/ce/monitor-client-api-c/AuthenticationInitException.h"
#include "glite/ce/monitor-client-api-c/AuthorizationException.h"
#include "glite/ce/monitor-client-api-c/SubscriptionNotFoundException.h"
//#include "glite/ce/monitor-client-api-c/CEMON_CLIENT.nsmap"

#include <string>
#include <vector>

extern "C" {
#undef IOV_MAX
#include "glite/security/glite_gsplugin.h"
};

namespace cemon_ex = glite::ce::monitor_client_api::cemon_exceptions;

namespace glite {
  namespace ce {
    namespace monitor_client_api {
      namespace soap_proxy {

/// Base class for all CEMonitor requests.
/**
	AbsRequest class is the base class of all remote call
	wrappers that realizes the client side of the CEMonitor.\n
	A remote call wrapper
	completely hides the user from the details of SOAP communication and does one or more remote calls
	to the remote web service (CEMonitor) using the SOAP transport protocol.
	AbsRequest provides some interfaces and implements some common utilities like
	setting and retrieve internal error messages hiding the user from the
	details of obtaining them from the SOAP data structure, tokenize a string etc. ...\n
	Direct instantiation of this class is possible but useless because it just
	initializes the SOAP runtine and allow the setting of some parameter but doesn't
	actually perform any remote call. The user has to
	instantiate the right class for the request he wants to send to the CEMonitor. See below for
	the list of these classes:
	\sa CEEvent
	\sa CEInfo
	\sa CEPing
	\sa CETopics
	\sa CESubscription
*/
class AbsRequest {

 private:
  //struct soap *SOAP;
  struct soap *SOAP;
  std::string  serviceURL;
  glite_gsplugin_Context ctx;
  cefaults__BaseFaultType _BaseFault;
  _monitorfaults__SubscriptionFault _SubscriptionFault;
  _monitorfaults__TopicNotSupportedFault _TopicNotSupportedFault;
  _monitorfaults__DialectNotSupportedFault _DialectNotSupportedFault;
  _cefaults__GenericFault _GenericFault;
  _cefaults__AuthenticationFault _AuthenticationFault;
  _cefaults__AuthorizationFault _AuthorizationFault;
  _monitorfaults__SubscriptionNotFoundFault _SubscriptionNotFoundFault;

  int BaseFault, SubscriptionFault, TopicNotSupportedFault, DialectNotSupportedFault, GenericFault, AuthenticationFault, AuthorizationFault, SubscriptionNotFoundFault;

protected:
  struct soap* getSoap() const throw() { return SOAP; }

  /**
   * No one can copy an AbsRequest object
   */
  AbsRequest(const AbsRequest&) {} 
  AbsRequest& operator=(const AbsRequest&) { return *this; }
  void         setError(void) throw();

  std::string  errorMessage;
  std::string  errorCode;

  void throwCEException(int) throw(cemon_ex::GenericException&, 
				   cemon_ex::TopicNotSupportedException&, 
				   cemon_ex::DialectNotSupportedException&, 
				   cemon_ex::SubscriptionException&,
				   cemon_ex::AuthenticationException&,
				   cemon_ex::AuthorizationException&,
				   cemon_ex::SubscriptionNotFoundException&);

  void throwAbsException() throw(cemon_ex::ServiceNotFoundException&,
			  	 cemon_ex::GeneralException&);

  void setNamespace() throw();
  void* getFault() throw();
  int getFaultType() throw();

public:
  /// Creates an AbsRequest object
  /**
	The default CTOR takes care of the SOAP runtime structures initialization.
  */
  AbsRequest() throw(cemon_ex::GeneralException&);
  /// Deletes an AbsRequest object
  /**
	The DTOR cleanly destroys the SOAP runtime data structures.
  */
  virtual ~AbsRequest(void) throw();
  /// Returns the error message
  /**
	A fault raised from the CEMonitor service and sent to the client via SOAP is usually composed by an error message and an error code. This method provides read access to the message.
  */
  std::string getErrorMessage(void) const throw() { return errorMessage; }
  /// Returns the error code
  /**
	A fault raised from the CEMonitor service and sent to the client via SOAP is usually composed by an error message and an error code. This method provides read access to the code.
  */
  std::string getErrorCode(void) const throw() { return errorCode; }
  /// Sets the service endpoint address
  /**
	Sets the address of the remote CEMonitor service. The format must be: http[s]://[hostname]:[tcp_port]/[service_context]. A CEMonitor address's format example is https://grid005.pd.infn.it:8443/ce-monitor/services/CEMonitor .
  */
  void setServiceURL(const std::string& url) throw() { serviceURL = url; }
  /// Cleans up SOAP stuff
  /**
	Cleans up all deserialized C++ data structures preventing memory leaks; another remote call can be
	safely done just after calling this method.
  */
  virtual void cleanup(void) throw();
  /// Returns the address of the remote CEMonitor service previously set.
  /** */
  std::string getServiceURL() const throw() { return serviceURL; }
  /// Tokenize a string
  /**
	Return a vector of strings by tokenizing the 1st argument string with the 2nd argument as tokens.
  */
      //std::vector<std::string> tokenize(const std::string&, const std::string&);
  void authenticate(const char*, const char*) throw(cemon_ex::AuthenticationInitException&);
};
      }
    }
  }
}

#endif
