/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CEEvent: header file for a C++ class wrapping the Ping call

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CEEvent.h,v 1.38 2007/12/14 11:02:37 adorigo Exp $
*/

#ifndef __GLITE_CE_MONITOR_CLIENT_CEEVENT_H__
#define __GLITE_CE_MONITOR_CLIENT_CEEVENT_H__

#include "glite/ce/monitor-client-api-c/AbsRequest.h"
#include "glite/ce/monitor-client-api-c/Topic.h"
#include <string>
#include <list>
#include <vector>

//#define GETEVENTRESP monitorws__GetEventResponse
//#define GETEVENTCALL soap_call_monitorws__GetEvent
//#define EVENT monitortypes__Event
//#define EVENT_FIELD Event

/// Wrapper class for getEvent request
/**
   This class is a wrapper of the getEvent remote call and hides the user from any communication and SOAP detail.\n
   It is not (yet) thread safe and can be used safely for multiple getEvent remote calls (remember to use AbsRequest::cleanup() before call it again and again)
*/

namespace cemon_ex = glite::ce::monitor_client_api::cemon_exceptions;

namespace glite {
namespace ce {
namespace monitor_client_api {
namespace soap_proxy {

class CEEvent : public AbsRequest {
  
 private:
  TOPIC *m_topic;

  monitorws__GetEventResponse m_evtResp;
  monitorws__GetTopicEventResponse m_topicEvtResp;
  
  monitortypes__Event* m_savedEvent;
  int m_sizeMessage, m_messageCounter;
  bool m_retrievedInfo;

 protected:
  CEEvent(const CEEvent&) {}
  CEEvent& operator=(const CEEvent&) { return *this; }

 public:
  /// Creates a CEEvent object
  /**
	Creates a CEEvent object and initializes the SOAP runtime for SSL authentication.
        \param certfile  The filename of the user's proxy certificate (voms or non-voms).
      	\param certpath  The name of the directory containing the certification authority's files for reverse AuthN (i.e. service authenticates itself in the client). For now can be any existing path (the reverse AuthN is not yet implemented).
	\throw AuthenticationException  If the certfile and/or certpath are missing on disk or not readable/accessible, or if the user's proxy certificate is corrupted or for some reason not accepted by gSOAP's internal SSL routines. AuthenticationException::what() gives the user the human readable reason of the exception.
  */
   CEEvent(const std::string& certfile, const std::string& certpath) throw(cemon_ex::AuthenticationInitException&);   

  /// Destroy the CEEvent object
  /**
	Simply calls AbsRequest::DTOR that cleanly destroys the SOAP runtime
  */
  virtual ~CEEvent(void) throw();

  /// Sets the requested event's parameter
  /**
	The CEMonitor getEvent remote call returns an event generated by a certain sensor. The specification of the sensor the user want the event of, is made by creating a Topic object and passing it as argument of this method. This method must be called before the first call of CEEvent::getEvent(), and can be called again before subsequent calls of CEEvent::getEvent().
	\sa Topic
	\param topic  The topic (sensor) generating the events the user is interested to
  */
  void setRequestParam(Topic *topic) throw();
  /// Invokes the remote call getEvent
  /**
	Invokes the CEMonitor remote call getEvent and put the received event into an internal array of strings that the user can browse by using CEEvent::getNextEventMessage().
	\sa CEEvent::getNextEventMessage()
	\throw GenericException&  When the CEMonitor service sends back to the client a generic fault;
	\throw TopicNotSupportedException&  When the user specified an un-existing topic name;
	\throw DialectNotSupportedException&  When the user asked for an event with an unsupported dialect language;
	\throw ServiceNotFoundException&  When the user specified an un-existing service's address (see AbsRequest::setServiceURL() for the correct address format). This exception can be raised if the host is unreachable/non-existent, or if the connection is refused (e.g. wrong tcp port), or if the root-context of the remote container (TomCAT) is wrong;
	\throw GeneralException&  If some other kind of error occurred.
	All raised exceptions contain human readable information the user can get using the method ::what().
  */
  void getEvent(void) throw(cemon_ex::GenericException&,
			    cemon_ex::TopicNotSupportedException&,
			    cemon_ex::DialectNotSupportedException&,
			    cemon_ex::SubscriptionException&,
			    cemon_ex::SubscriptionNotFoundException&,
			    cemon_ex::ServiceNotFoundException&,
			    cemon_ex::GeneralException&,
			    cemon_ex::AuthenticationException&,
			    cemon_ex::AuthorizationException&) ;

  void getTopicEvents(std::list< std::vector<std::string> >& ) throw(cemon_ex::GenericException&,
			  	  cemon_ex::TopicNotSupportedException&,
				    cemon_ex::DialectNotSupportedException&,
				    cemon_ex::SubscriptionException&,
			    cemon_ex::SubscriptionNotFoundException&,
			    cemon_ex::ServiceNotFoundException&,
			    cemon_ex::GeneralException&,
			    cemon_ex::AuthenticationException&,
			    cemon_ex::AuthorizationException&) ;

  //bool nextEvent(void);

  /// Gets the ID of last event
  /**
	Returns the ID (integer) of last retrieved event.
	\return The integer ID of last event
  */
  int getEventID() const throw();
  
  /// Gets the timestamp of last event
  /**
	Returns the timestamp (expressed as an integer indicating the number of second since the "epoch") of last retrieved event.
	\return The last event's timestamp
  */
  time_t getEventTimestamp() const throw();
  /// Gets the event's timestamp
  /**
	Returns the human readable timestamp of last retrieved event.
	\return The last event's timestamp in human readable format
  */
  std::string getEventDateTime() const throw();
  /// Gets the event's producer
  /**
	Returns the name of the sensor that produced last retrieved event
	\return The name of the last event producer
  */
  std::string getEventProducer() const throw();
  /// Returns the next event message
  /**
	This method returns the next event message stored in an array consisting of
	CEEvent::getNumberOfMessages() messages retrieved by a previous call of
	CEEvent::getEvent(). When the last message is reached, a subsequent call
	of this method returns NULL.
  */
  const char* getNextEventMessage() throw();
  /// Returns the last event message
  /**
	Returns the last event message stored in an array consisting of
	CEEvent::getNumberOfMessages() messages retrieved by a previous call of
	CEEvent::getEvent().
  */
  std::string getLastEventMessage() const throw() ;
  /// Returns a particular message
  /**
	Returns the message at position index from the array consisting of
	CEEvent::getNumberOfMessages() messages retrieved by a previous call of
	CEEvent::getEvent().
  */
  std::string getEventMessageAt(int index) const throw();
  /// Returns the number of event messages
  /**
	Returns the number of messages stored in an array consisting of
	CEEvent::getNumberOfMessages() messages retrieved by a previous call of
	CEEvent::getEvent().
  */
  int         getNumberOfMessages() const throw();
};
}
}
}
}
#endif
