
/*
        Copyright (c) 2004 on behalf of the EU EGEE Project:
        The European Organization for Nuclear Research (CERN),
        Istituto Nazionale di Fisica Nucleare (INFN), Italy
        Datamat Spa, Italy
        Centre National de la Recherche Scientifique (CNRS), France
        CS Systeme d'Information (CSSI), France
        Royal Institute of Technology, Center for Parallel Computers (KTH-PDC), Sweden
        Universiteit van Amsterdam (UvA), Netherlands
        University of Helsinki (UH.HIP), Finland
        University of Bergen (UiB), Norway
        Council for the Central Laboratory of the Research Councils (CCLRC), United Kingdom

        CEInfo: header file for a C++ class wrapping the GetInfo  call

        Authors: Alvise Dorigo <alvise.dorigo@pd.infn.it>
        Version info: $Id: CEInfo.h,v 1.30 2007/12/14 11:02:37 adorigo Exp $
*/

#ifndef __GLITE_CE_MONITOR_CLIENT_CEINFO__
#define __GLITE_CE_MONITOR_CLIENT_CEINFO__

#include "glite/ce/monitor-client-api-c/AbsRequest.h"
#include "glite/ce/monitor-client-api-c/Topic.h"
#include "glite/ce/monitor-client-api-c/Action.h"
#include <string>
#include <vector>

#define INFO monitortypes__Info
#define GETINFORESP monitorws__GetInfoResponse
#define INFO_FIELD Info
#define GETINFO soap_call_monitorws__GetInfo

namespace cemon_ex = glite::ce::monitor_client_api::cemon_exceptions;

namespace glite {
  namespace ce {
    namespace monitor_client_api {
      namespace soap_proxy {

/// Wrapper class for getInfo request
/**
	This class is a wrapper of the getInfo remote call and hides the user from any communication and SOAP detail.\n
	It is not (yet) thread safe and can be used safely for multiple getInfo remote calls (remember to use AbsRequest::cleanup() before call it again and again). The information is composed by a Version and a Description of the remote service, a list of supported topics (sensors of the CEMonitor) and a list of supported actions. See CEMonitor Documentation for more details about topics and actions.
*/
class CEInfo : public AbsRequest {

 private:
  INFO info;
  GETINFORESP resp;
  void init() throw();

 protected:
  CEInfo(const CEInfo&) {}
  CEInfo& operator=(const CEInfo&) { return *this; }

 public:
  /// Creates a CEInfo object
  /**
	Creates a CEInfo object and initializes the SOAP runtime for SSL authentication.
        \param certfile  The filename of the user's proxy certificate (voms or non-voms).
      	\param certpath  The name of the directory containing the certification authority's files for reverse AuthN (i.e. service authenticates itself in the client). For now can be any existing path (the reverse AuthN is not yet implemented).
	\throw AuthenticationException  If the certfile and/or certpath are missing on disk or not readable/accessible, or if the user's proxy certificate is corrupted or for some reason not accepted by gSOAP's internal SSL routines. AuthenticationException::what() gives the user the human readable reason of the exception.
  */
   CEInfo(const std::string certfile, const std::string certpath) throw(cemon_ex::AuthenticationInitException&);
  /// Destroy the CEInfo object
  /**
	Simply calls AbsRequest::DTOR that cleanly destroys the SOAP runtime
  */
  virtual ~CEInfo(void) throw();

  /// Invokes the remote call getInfo
  /**
	Invokes the CEMonitor remote call getInfo and memorizes the received informations.\n
	The user can get these infos using the methods CEInfo::getVersion(), CEInfo::getDescription, CEInfo::getTopics(...), CEInfo::getActions(...).\n All raised exceptions contain human readable information the user can get using the method ::what().
	\throw GenericException&  When the CEMonitor service sends back to the client a generic fault
	\throw ServiceNotFoundException&  When the user specified an un-existing service's address (see AbsRequest::setServiceURL() for the correct address format). This exception can be raised if the host is unreachable/non-existent, or if the connection is refused (e.g. wrong tcp port), or if the root-context of the remote container (TomCAT) is wrong.
	\throw GeneralException&  If some other kind of error occurred.
	All raised exceptions contain human readable information the user can get using the method ::what().

  */
  void        getInfo() throw(cemon_ex::GenericException&, 
			      cemon_ex::TopicNotSupportedException&, 
			      cemon_ex::DialectNotSupportedException&,
			      cemon_ex::SubscriptionException&,
			      cemon_ex::SubscriptionNotFoundException&,
			      cemon_ex::ServiceNotFoundException&,
			      cemon_ex::GeneralException&,
			      cemon_ex::AuthenticationException&,
			      cemon_ex::AuthorizationException&);
			      
  /// Returns the remote service's version
  std::string getVersion() const throw();
  /// Returns the remote service's description
  std::string getDescription() const throw();
  /// Gets the service's topics
  /**
	This method provides all the topics (sensors) supported by the CEMonitor.\n
	They are put in the user provided (and allocated) pointer to vector of pointers to Topic objects.\n
	For each topic object the user can browse it invoking the methods of Topic class.
	\sa Topic
  */
  void getTopics(std::vector<Topic*> *) const throw();
  /// Gets the service's actions
  /**
	This method provides all the actions supported by the CEMonitor.\n
	They are put in the user provided (and allocated) pointer to vector of pointers to Action objects.\n
	For each action object the user can browse it invoking the methods of ActionW class.
	\sa ActionW
  */
  void getActions(std::vector<ActionW*> *) const throw();
  
  std::string getDN( void ) throw();
};
      }
    }
  }
}


#endif
